#ifndef _include_bleifrei_gfx_Effect_hpp_
#define _include_bleifrei_gfx_Effect_hpp_ 
//----------------------------------------------------------------------------------------------------------------------
#include <vector>
#include <string>
//----------------------------------------------------------------------------------------------------------------------
#include "../math/Matrix.hpp"
#include "../Base.hpp"
//----------------------------------------------------------------------------------------------------------------------
#include "CgFX.hpp"
//----------------------------------------------------------------------------------------------------------------------
namespace bleifrei {
	namespace tex {
		class Texture;
	}
	namespace gfx {
//----------------------------------------------------------------------------------------------------------------------

/*		enum ParameterType
		{
			ParameterTypeUnknown		= CgFXPT_UNKNOWN,
			ParameterTypeBool			= CgFXPT_BOOL,
			ParameterTypeInt			= CgFXPT_INT,
			ParameterTypeFloat			= CgFXPT_FLOAT,
			ParameterTypeDWord			= CgFXPT_DWORD,
			ParameterTypeString			= CgFXPT_STRING,

			// Ref counted objects; Can be accessed as objects or as strings
			ParameterTypeTexture		= CgFXPT_TEXTURE,
			ParameterTypeCubeTexture	= CgFXPT_CUBETEXTURE,
			ParameterTypeVolumeTexture	= CgFXPT_VOLUMETEXTURE,
			ParameterTypeVertexShader	= CgFXPT_VERTEXSHADER,
			ParameterTypePixelShader	= CgFXPT_PIXELSHADER,

			// Complex type
			ParameterTypeStruct			= CgFXPT_STRUCT,
		    
			ParameterTypeMax
		};
*/

//----------------------------------------------------------------------------------------------------------------------

class Parameter
{
	friend class Effect;
public:
	~Parameter(void)
	{
	}

	std::string getName(void) const                 { return cgGetParameterName(m_cgParameter); }
	std::string getSemantic(void) const             { return cgGetParameterSemantic(m_cgParameter); }

	CGparameter cg_getParameter() const             { return m_cgParameter; }

protected:
	Parameter(CGparameter cgParameter) :
		m_cgParameter(cgParameter)
	{
	}

	CGparameter m_cgParameter;
};

//----------------------------------------------------------------------------------------------------------------------

class Pass
{
	friend class Effect;
public:
	~Pass(void)
	{
	}

	std::string getName(void) const                 { return cgGetPassName(m_cgPass); }

protected:
	Pass(CGpass cgPass) :
		m_cgPass(cgPass)
		{
		}

	CGpass m_cgPass;
};

//----------------------------------------------------------------------------------------------------------------------

class Technique
{
	friend class Effect;
public:
	typedef std::vector<Pass>::iterator pass_iterator;

	~Technique(void)
	{
	}

	pass_iterator passBegin(void)                   { return m_passes.begin(); }
	pass_iterator passEnd(void)                     { return m_passes.end(); }

	std::string getName(void) const                 { return cgGetTechniqueName(m_cgTechnique); }
	unsigned int getPassCount(void) const						{ return (unsigned int)m_passes.size(); }

	pass_iterator getPassByName(const std::string &name);

protected:
	Technique(CGtechnique cgTechnique) :
		m_cgTechnique(cgTechnique)
	{
	}

	CGtechnique m_cgTechnique;
	std::vector<Pass> m_passes;
};

//----------------------------------------------------------------------------------------------------------------------

class Effect :
	public Base
{
	friend class EffectManager;
public:
	typedef std::vector<Technique>::iterator technique_iterator;
	typedef std::vector<Parameter>::iterator parameter_iterator;

	technique_iterator techniqueBegin(void)         { return m_techniques.begin();	}
	technique_iterator techniqueEnd(void)           { return m_techniques.end();	}

	parameter_iterator parameterBegin(void)         { return m_parameters.begin();	}
	parameter_iterator parameterEnd(void)           { return m_parameters.end();	}

	bool setTechnique(Technique &technique);

	bool setParameter(Parameter &parameter, const float value);
	bool setParameter(Parameter &parameter, const math::Vector3f &vector);
	bool setParameter(Parameter &parameter, const math::Vector4f &vector);
	bool setParameter(Parameter &parameter, const math::Matrix4x4f &matrix);
	bool setParameter(Parameter &parameter, tex::Texture *texture);

	technique_iterator getTechniqueByName(const std::string &name);
	parameter_iterator getParameterByName(const std::string &name);
	parameter_iterator getParameterBySemantic(const std::string &semantic);

	bool begin(unsigned int &passes);
	void end(void);

	bool beginPass(unsigned int pass);
	void endPass();

	// Cg specific
	CGeffect cg_getEffect(void)                     { return m_cgEffect; }

protected:
	bool load(const std::string &filename);
	void unload(void);

	Effect(void);
	~Effect(void);

	CGeffect m_cgEffect;
	std::vector<Parameter> m_parameters;
	std::vector<Technique> m_techniques;
	Technique *currentTechnique;
	unsigned int m_currentPass;
};

//----------------------------------------------------------------------------------------------------------------------
	}
}
//----------------------------------------------------------------------------------------------------------------------
#endif //_include_bleifrei_gfx_Effect_hpp_
