
#ifndef _include_bleifrei_gfx_Font_hpp_
#define _include_bleifrei_gfx_Font_hpp_ 


// includes
#include <string>

namespace bleifrei {
	namespace io {
		class ConfigFile;
	}
	namespace tex {
		class Texture;
	}
	namespace gfx {

		class Font {
		public:
			Font(io::ConfigFile &config_file);
			~Font(void);

			// methods
			void printf(int x, int y, float scale, const char *str, ...);
			void print(int x, int y, float scale, int numchars, const char *str);
			void print(int x, int y, float scale, const std::string &str);

			int getMaxChars(const char *str, int numchars, int maxWidth);
			int getMaxChars(const std::string &str, int maxWidth);
			int getWidth(const char *str, int numchars);
			int getWidth(const std::string &str);
			int getHeight() const											{ return m_height;				}
			const std::string getName(void) const							{ return m_name;				}

		protected:

			// attrubutes
			std::string m_name;

			tex::Texture *m_texture;

			unsigned int m_charBase;
			unsigned char m_width[256];
			int m_height;
		};

		inline void Font::print(int x, int y, float scale, const std::string &str)
		{
			print(x, y, scale, (int)str.length(), str.c_str());
		}

		inline int Font::getMaxChars(const std::string &str, int maxWidth)
		{
			return getMaxChars(str.c_str(), (int)str.length(), maxWidth);
		}

		inline int Font::getWidth(const std::string &str)
		{
			return getWidth(str.c_str(), (int)str.length());
		}

	}
}


#endif //_include_bleifrei_gfx_Font_hpp_
