/** 
 * bleifrei utility library
 *
 * Copyright (C) 2003 Ulf Borgenstam, Jonas Svensson.
 *
 * Do not distribute. If you know someone interested in obtaining these sources
 * please have them contact us.
 */

#ifndef _include_bleifrei_io_io_hpp_
#define _include_bleifrei_io_io_hpp_ 


// includes
#include <iostream>
#include <string>

#include "../platform/types.hpp"

namespace bleifrei {
	namespace io {

		// $TODO: move to file format header
		#pragma pack(push, 1)

		typedef platform::dword FileId;

		struct FileVersion
		{
			platform::word major;
			platform::word minor;
		};

		#pragma pack(pop)

		template<class T>
		inline void
		get(std::istream &is, T &t)
		{
			is.read((char *)&t, sizeof(t));
		}

		template<class T>
		inline void
		put(std::ostream &os, const T &t)
		{
			os.write((const char *)&t, sizeof(t));
		}

		inline void
		get(std::istream &is, std::string &str)
		{
			platform::dword length;
			get(is, length);
			
			if (length > 0)
			{
				char *buffer = new char[length + 1];
				is.read(buffer, length + 1);
				str = buffer;
				delete[] buffer;
			}
			else
			{
				str = "";
			}
		}

		inline void
		put(std::ostream &os, const std::string &str)
		{
			platform::dword length = (platform::dword)str.size();
			
			put(os, length);
			if (length > 0)
			{
				os.write(str.c_str(), length + 1);
			}
		}

		inline FileId
		string2id(const char *str)
		{
			return *((FileId *)str);
		}

	}
}


#endif //_include_bleifrei_io_io_hpp_ 
