/** 
 * bleifrei utility library
 *
 * Copyright (C) 2003 Ulf Borgenstam, Jonas Svensson.
 *
 * Do not distribute. If you know someone interested in obtaining these sources
 * please have them contact us.
 */

// includes
#include "../io/io.hpp"
#include "../platform/types.hpp"

#include "Mesh.hpp"
#include "Scene.hpp"

using namespace bleifrei::io;
using namespace bleifrei::platform;
using namespace std;

namespace bleifrei {
	namespace scene {

		#pragma pack(push, 1)

		struct SceneHeader
		{
			dword materials;
			dword meshes;
		};

		#pragma pack(pop)

		Scene::Scene(void)
		{
		}

		Scene::~Scene(void)
		{
			vector<Mesh *>::iterator it;
			for (it = meshes.begin(); it != meshes.end(); ++it)
			{
				safe_release(*it);
			}
		}

		void
		Scene::insert(Mesh *mesh)
		{
			mesh->addReference();
			meshes.push_back(mesh);
		}

		unsigned int
		Scene::add(Material &material)
		{
			materials.push_back(material);
			return (unsigned int)(materials.size() - 1);
		}

		void
		Scene::read(istream &is)
		{
			FileId id;
			get(is, id);

			if (!is)
			{
				// $TODO: signal error "Failed to load file id!"
				return;
			}

			if (id != string2id("SCEN"))
			{
				// $TODO: signal error "Not a valid scene file!"
				return;
			}

			FileVersion ver;
			get(is, ver);

			if (ver.major != 1 || ver.minor != 0)
			{
				// $TODO: signal error "Unknown scene version!"
				return;
			}

			SceneHeader header;
			get(is, header);									

			for (unsigned int i = 0; i < header.materials; i++)
			{
				Material material;

				// $TODO: catch exceptions
				material.read(is);
				materials.push_back(material);
			}

			for (unsigned int i = 0; i < header.meshes; i++)
			{
				Mesh *mesh = new Mesh();

				if (mesh)
				{
					// $TODO: catch exceptions
					mesh->read(is);
					meshes.push_back(mesh);
				}
				else
				{
					// $TODO: signal "Out of memory"
				}
			}
		}

		void
		Scene::write(ostream &os)
		{
			FileId id = string2id("SCEN");
			put(os, id);

			FileVersion ver;
			ver.major =	1;
			ver.minor = 0;
			put(os, ver);

			SceneHeader header;
			header.materials = (dword)materials.size();
			header.meshes = (dword)meshes.size();
			put(os, header);

			for (vector<Material>::iterator it = materials.begin(); it != materials.end(); ++it)
			{
				it->write(os);
			}

			for (vector<Mesh *>::iterator it = meshes.begin(); it != meshes.end(); ++it)
			{
				(*it)->write(os);
			}
		}
	}
}
