
// includes
#include "bleifrei/gfx/Font.hpp"
#include "bleifrei/gfx/opengl.hpp"
#include "bleifrei/tex/Texture.hpp"

#include "FXWindow.hpp"

using namespace bleifrei::gfx;
using namespace bleifrei::tex;
using namespace bleifrei;

extern int g_width;
extern int g_height;

FXWindow::FXWindow(Effect *effect, Font *font, int width, int height, int x, int y) :
	m_effect(0)
{
	move(x, y);
	resize(width, height);
	calculatePosition(g_width, g_height);
	m_texture = new TextureRect();
	m_firstFrameTexture = new TextureRect();
	m_font = font;
	setEffect(effect);
}

FXWindow::~FXWindow(void)
{
	safe_release(m_effect);
	safe_release(m_texture);
	safe_release(m_firstFrameTexture);
}

void
FXWindow::setEffect(Effect *effect)
{
	safe_release(m_effect);
	m_effect = effect;

	if (m_effect)
	{
		m_framebufferParam = m_effect->getParameterBySemantic("FRAME_BUFFER_TEXTURE");
		m_firstFrameParam = m_effect->getParameterBySemantic("FIRST_FRAME_TEXTURE");
		m_windowPosition = m_effect->getParameterBySemantic("WINDOW_POSITION");
		m_technique = m_effect->techniqueBegin();
    if (m_technique != m_effect->techniqueEnd())
		{
			m_effect->addReference();
		}
		else
		{
			m_effect = 0;
		}
	}
}

void
FXWindow::onMouseDown(int x, int y)
{
	if (m_effect && m_technique != m_effect->techniqueEnd())
	{
		if (y < 10)
		{
            if (++m_technique == m_effect->techniqueEnd())
			{
				m_technique = m_effect->techniqueBegin();
			}
		}
	}
	if (x > getWidth() - 5)
	{
		m_resizeX = true;
	}
	else
	{
		m_resizeX = false;
	}

	if (y > getHeight() - 5)
	{
		m_resizeY = true;
	}
	else
	{
		m_resizeY = false;
	}

	lastX = x;
	lastY = y;
}

void
FXWindow::onMouseMove(int x, int y)
{
	if (isClicked())
	{
		if (!m_resizeX && !m_resizeY)
		{
			move(getX() + x - lastX, getY() + y - lastY);
		}
		else
		{
			int w = getWidth();
			int h = getHeight();
			if (m_resizeX)
			{
				w += x - lastX;
				lastX = x;

				if (w < 100)
				{
					m_resizeX = false;
					w = 100;
				}
			}
			if (m_resizeY)
			{
				h += y - lastY;
				lastY = y;

				if (h < 10)
				{
					m_resizeY = false;
					h = 10;
				}
			}
			resize(w, h);
		}
	}
}

void
FXWindow::onPaintBackground(int absx, int absy)
{
	if (!m_effect)
	{
		return;
	}

	m_effect->setTechnique(*m_technique);

	unsigned int passes;
	m_effect->begin(passes);
	{
		for (unsigned int pass = 0; pass < passes; ++pass)
		{
			if (pass == 0)
			{
				m_firstFrameTexture->loadFromFramebuffer(absx, g_height - absy - getHeight(), getWidth(), getHeight());
			}
			m_texture->loadFromFramebuffer(absx, g_height - absy - getHeight(), getWidth(), getHeight());
			if (m_framebufferParam != m_effect->parameterEnd())
			{
				m_effect->setParameter(*m_framebufferParam, m_texture);
			}
			if (m_firstFrameParam != m_effect->parameterEnd())
			{
				m_effect->setParameter(*m_firstFrameParam, m_firstFrameTexture);
			}
			if (m_windowPosition != m_effect->parameterEnd())
			{
				m_effect->setParameter(*m_windowPosition, math::Vector4f((float)absx, (float)(g_height - absy - getHeight()), (float)getWidth(), (float)getHeight()));
			}

			m_effect->beginPass(pass);

			glBegin(GL_QUADS);
			{
				glTexCoord2i(0, getHeight());
				glVertex2i(absx, absy);
				glTexCoord2i(0, 0);
				glVertex2i(absx, absy + getHeight());
				glTexCoord2i(getWidth(), 0);
				glVertex2i(absx + getWidth(), absy + getHeight());
				glTexCoord2i(getWidth(), getHeight());
				glVertex2i(absx + getWidth(), absy);
			}
			glEnd();

			m_effect->endPass();
		}
	}
	m_effect->end();

	nullTexture->forceBind();
	glColor4f(192 / 255.0f, 192 / 255.0f, 192 / 255.0f, 1);
	glBegin(GL_LINE_LOOP);
	{
		glVertex2i(absx, absy);
		glVertex2i(absx, absy + getHeight());
		glVertex2i(absx + getWidth(), absy + getHeight());
		glVertex2i(absx + getWidth(), absy);
	}
	glEnd();

	m_font->print(absx + 2, absy + 2, 1.0f, "next technique >");

	glColor3f(98 / 255.0f, 142 / 255.0f, 181 / 255.0f);
	m_font->printf(absx + 2, absy - m_font->getHeight() - 2, 1.0f, "technique: %s", m_technique->getName().c_str());
}
